clear
%% parameters
alpha = 1;
beta = 0.5;
rho = 0.02;
sigma = 0.6;
t = 0; % initial time is zero
tau = 0:1:25;
% create time vector
time = tau-t;
%% 4. compute the discount functions
expoDist = exp(-rho*(tau-t));
hyperDist = (1+alpha*(tau-t)).^(-rho/alpha);
quasiDist = beta*exp(-rho*(tau-t));
quasiDist(1)  = 1;

figure('name','discount functions')
plot(time, expoDist, 'k', time, hyperDist, 'b', time, quasiDist, 'c')
legend('expo','hyper','quasi')

%% 5. compute discounted utility over time
c = tau; % assume consumption is linear in time
u = (c.^(1-sigma)-1)/(1-sigma);
discountedU_expo = expoDist.*u; 
discountedU_hyper = hyperDist.*u; 
discountedU_quasi = quasiDist.*u; 

figure('name','discounted utility function')
plot(time, discountedU_expo, 'k', time, discountedU_hyper , 'b', time, discountedU_quasi, 'c')
legend('expo','hyper','quasi')




%% 6. same level of discounted utility implies same level of discount function
tau=25; % target period
%targetYear = 25;
targetU = 0.5;
c25 = tau; % as c=tau
u25 = (c25^(1-sigma)-1)/(1-sigma);
targetDiscount= targetU/u25; % discount function at tau=25
rhoExpo = -log(targetDiscount)/(tau-t); 
rhoHyper = -alpha*log(targetDiscount)/log(1+alpha*(tau-t)); 
rhoQuasi = -log(targetDiscount/beta)/(tau-t); 

% compute discounted utility
tau=0:1:25;
expoDist = exp(-rhoExpo*(tau-t));
hyperDist = (1+alpha*(tau-t)).^(-rhoHyper/alpha);
quasiDist = beta*exp(-rhoQuasi*(tau-t));
quasiDist(1)  = 1;
discountedU_expo = expoDist.*u; 
discountedU_hyper = hyperDist.*u; 
discountedU_quasi = quasiDist.*u; 

lifetimeExpo = sum(discountedU_expo);
lifetimeHyper = sum(discountedU_hyper);
lifetimeQuasi = sum(discountedU_quasi);

if lifetimeQuasi>lifetimeHyper 
    disp('next')
    if lifetimeQuasi>lifetimeExpo 
        fprintf('the quasi-hyperbolic discount function gives the highest lifetime utility of %g\n',lifetimeQuasi )
    else
        fprintf('the exponential discount function gives the highest lifetime utility of %g\n', lifetimeExpo)
    end
else
    disp('next')
    if lifetimeHyper>lifetimeExpo 
        fprintf('the hyperbolic discount function gives the highest lifetime utility of %g\n', lifetimeHyper)
    else
        fprintf('the exponential discount function gives the highest lifetime utility of %g\n', lifetimeExpo)
    end
    
end


figure('name','discounted utility function new')
plot(time, discountedU_expo, 'k', time, discountedU_hyper , 'b', time, discountedU_quasi, 'c')
legend('expo','hyper','quasi')
ylabel('discounted utility')
xlabel('time')
